package com.example.demo.controllers;

import com.example.demo.exception.ApiException;
import com.example.demo.mappers.AccountMapper;
import com.example.demo.models.Account;
import com.example.demo.models.dto.AccountDTO;
import com.example.demo.models.dto.AuthRequestDTO;
import com.example.demo.models.dto.UpdateAccountDTO;
import com.example.demo.security.jwt.JwtUtil;
import com.example.demo.services.impl.AccountServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.web.bind.annotation.*;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Account")
public class AccountController {

    private final AccountServiceImpl accountsService;
    private final AuthenticationManager authenticationManager;
    private final AccountMapper mapper;
    private final JwtUtil jwtUtil;

    @GetMapping("/Me")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<AccountDTO> me() {
        Account account = accountsService.getAuthenticated();
        return ResponseEntity.ok(mapper.mapShort(account));
    }

    @PostMapping("/SignUp")
    public ResponseEntity<Void> signUp(@RequestBody AuthRequestDTO request) {
        Account account = mapper.map(request);
        account.setRole("ROLE_USER");
        accountsService.register(account);
        return new ResponseEntity<>(null, HttpStatus.CREATED);
    }

    @PostMapping("/SignIn")
    public ResponseEntity<String> signIn(@RequestBody AuthRequestDTO request) {
        UsernamePasswordAuthenticationToken authenticationToken =
                new UsernamePasswordAuthenticationToken(request.getEmail(), request.getPassword());

        try {
            authenticationManager.authenticate(authenticationToken);
        } catch (BadCredentialsException e) {
            throw new ApiException("Неправильные логин или пароль");
        }

        String token = jwtUtil.generateToken(request.getEmail());

        return new ResponseEntity<>(token, HttpStatus.OK);
    }

    @PutMapping("/Update")
    public ResponseEntity<Void> update(@RequestBody UpdateAccountDTO request) {
        accountsService.updatePassword(request.getPassword());
        return ResponseEntity.ok().build();
    }
}
