package com.example.demo.controllers;


import com.example.demo.mappers.AccountMapper;
import com.example.demo.models.Account;
import com.example.demo.models.dto.AccountDTO;
import com.example.demo.models.dto.CreateAccountAdminDTO;
import com.example.demo.services.impl.AdminAccountServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Admin/Account")
public class AdminAccountController {

    private final AdminAccountServiceImpl service;
    private final AccountMapper accountMapper;

    @GetMapping("/{id}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<AccountDTO> getById(@PathVariable Long id) {
        Account account = service.getById(id);
        return ResponseEntity.ok(accountMapper.mapShort(account));
    }

    @PostMapping
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> create(@RequestBody CreateAccountAdminDTO createAccountAdminDTO) {
        Account account = new Account(
                createAccountAdminDTO.getEmail(),
                createAccountAdminDTO.getPassword(),
                createAccountAdminDTO.getAdmin() ? "ROLE_ADMIN" : "ROLE_USER"
        );

        service.register(account, createAccountAdminDTO.getDiskSize());

        return ResponseEntity.ok().build();
    }

    @PutMapping("/{userId}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> edit(@PathVariable Long id, @RequestBody CreateAccountAdminDTO createAccountAdminDTO) {
        Account account = new Account(
                createAccountAdminDTO.getEmail(),
                createAccountAdminDTO.getPassword(),
                createAccountAdminDTO.getAdmin() ? "ROLE_ADMIN" : "ROLE_USER"
        );

        service.edit(id, account, createAccountAdminDTO.getDiskSize());

        return ResponseEntity.ok().build();
    }

    @DeleteMapping("/{userId}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> delete(@PathVariable Long id) {
        service.delete(id);
        return ResponseEntity.ok().build();
    }
}
