package com.example.demo.controllers;

import com.example.demo.models.Directory;
import com.example.demo.models.dto.CreateDirectoryRequestDTO;
import com.example.demo.models.dto.DirectoryDTO;
import com.example.demo.services.impl.AdminDirectoryServiceImpl;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Admin/Directory")
public class AdminDirectoryController {

    private final AdminDirectoryServiceImpl directoryService;
    private final ModelMapper modelMapper;

    @GetMapping("/GetRoot/{userId}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<DirectoryDTO> getRoot(@PathVariable Long userId) {
        Directory directory = directoryService.findRoot(userId);
        return ResponseEntity.ok(modelMapper.map(directory, DirectoryDTO.class));
    }

    @GetMapping("/{id}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<DirectoryDTO> getDirectoryById(@PathVariable Long id) {
        Directory directory = directoryService.findDirectoryById(id);
        return ResponseEntity.ok(modelMapper.map(directory, DirectoryDTO.class));
    }

    @GetMapping("/GetByDirectory/{dirId}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<List<DirectoryDTO>> getChildren(@PathVariable Long dirId) {
        List<DirectoryDTO> dtos = directoryService.findChildren(dirId)
                .stream()
                .map(d -> modelMapper.map(d, DirectoryDTO.class))
                .toList();

        return ResponseEntity.ok(dtos);
    }

    @PostMapping
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> create(@RequestBody CreateDirectoryRequestDTO requestDTO) {
        directoryService.createDirectory(requestDTO.getName(), requestDTO.getParentId());
        return ResponseEntity.ok().build();
    }

    @PutMapping("/{id}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> update(@PathVariable Long id, @RequestBody CreateDirectoryRequestDTO requestDTO) {
        directoryService.updateDirectory(id, requestDTO.getName(), requestDTO.getParentId());
        return ResponseEntity.ok().build();
    }

    @DeleteMapping("/{id}")
    @Secured("ROLE_ADMIN")
    public ResponseEntity<Void> delete(@PathVariable Long id) {
        directoryService.deleteDirectory(id);
        return ResponseEntity.ok().build();
    }
}
