package com.example.demo.controllers;

import com.example.demo.models.Directory;
import com.example.demo.models.dto.CreateDirectoryRequestDTO;
import com.example.demo.models.dto.DirectoryDTO;
import com.example.demo.services.impl.DirectoryServiceImpl;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Directory")
public class DirectoryController {

    private final DirectoryServiceImpl directoryService;
    private final ModelMapper modelMapper;

    @GetMapping("/GetRoot")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<DirectoryDTO> getRoot() {
        Directory directory = directoryService.findRoot();
        return ResponseEntity.ok(modelMapper.map(directory, DirectoryDTO.class));
    }

    @GetMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<DirectoryDTO> getById(@PathVariable Long id) {
        Directory directory = directoryService.findById(id);
        return ResponseEntity.ok(modelMapper.map(directory, DirectoryDTO.class));
    }

    @GetMapping("/GetByDirectory/{dirId}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<List<DirectoryDTO>> getChildren(@PathVariable Long dirId) {
        List<DirectoryDTO> dtos = directoryService.findChildren(dirId)
                .stream()
                .map(el -> modelMapper.map(el, DirectoryDTO.class))
                .toList();

        return ResponseEntity.ok(dtos);
    }

    @PostMapping
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> create(@RequestBody CreateDirectoryRequestDTO requestDTO) {
        directoryService.createDirectory(requestDTO.getName(), requestDTO.getParentId());
        return ResponseEntity.ok().build();
    }


    @PutMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> update(@PathVariable Long id, @RequestBody CreateDirectoryRequestDTO requestDTO) {
        directoryService.updateDirectory(id, requestDTO.getName(), requestDTO.getParentId());
        return ResponseEntity.ok().build();
    }

    @DeleteMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> delete(@PathVariable Long id) {
        directoryService.deleteDirectory(id);
        return ResponseEntity.ok().build();
    }
}
