package com.example.demo.controllers;

import com.example.demo.models.File;
import com.example.demo.models.dto.CreateFileDTO;
import com.example.demo.models.dto.FileDTO;
import com.example.demo.services.impl.FileServiceImpl;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Files")
public class FileController {

    private final FileServiceImpl fileService;
    private final ModelMapper modelMapper;

    @GetMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<FileDTO> getById(@PathVariable Long id, @RequestParam String password) {
        File file = fileService.getById(id, password);
        return ResponseEntity.ok(modelMapper.map(file, FileDTO.class));
    }

    @GetMapping("/GetByDirectory/{dirId}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<List<FileDTO>> getChildren(@PathVariable Long dirId) {
        List<FileDTO> files = fileService.getChildren(dirId)
                .stream()
                .map(f -> modelMapper.map(f, FileDTO.class))
                .toList();

        return ResponseEntity.ok(files);
    }

    @PostMapping
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> createFile(@RequestBody CreateFileDTO dto) {
        fileService.createFile(
                dto.getName(),
                dto.getDirectoryId(),
                dto.getPassword(),
                dto.getShared()
        );

        return ResponseEntity.ok().build();
    }

    @PutMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> updateFileInfo(@PathVariable Long id, @RequestBody CreateFileDTO dto) {
        fileService.updateFileInfo(
                id,
                dto.getName(),
                dto.getDirectoryId(),
                dto.getPassword(),
                dto.getShared()
        );

        return ResponseEntity.ok().build();
    }

    @DeleteMapping("/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> delete(@PathVariable Long id) {
        fileService.delete(id);
        return ResponseEntity.ok().build();
    }
}
