package com.example.demo.controllers;

import com.example.demo.models.Account;
import com.example.demo.models.dto.StorageInfoResponseDTO;
import com.example.demo.services.impl.AccountServiceImpl;
import com.example.demo.services.impl.FileServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Storage")
public class StorageController {

    private final AccountServiceImpl accountService;
    private final FileServiceImpl fileService;

    @GetMapping("/Info")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<StorageInfoResponseDTO> getInfo() {
        Account account = accountService.getAuthenticated();
        return ResponseEntity.ok(
                new StorageInfoResponseDTO("Осталось " + account.getStorage().getFreeSpaceInMB() + " MB")
        );
    }

    @GetMapping("/Download/{id}")
    public ResponseEntity<Resource> downloadFile(@PathVariable Long id, @RequestParam String password, HttpServletRequest request) {

        Resource resource = fileService.loadById(id, password);

        String contentType = null;
        try {
            contentType = request.getServletContext().getMimeType(resource.getFile().getAbsolutePath());
        } catch (IOException ex) {
            log.info("Could not determine file type.");
        }

        if(contentType == null) {
            contentType = "application/octet-stream";
        }

        return ResponseEntity.ok()
                .contentType(MediaType.parseMediaType(contentType))
                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + resource.getFilename() + "\"")
                .body(resource);
    }

    @PutMapping("/Upload/{id}")
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity<Void> saveFile(@PathVariable Long id, @RequestPart MultipartFile file) {
        fileService.saveFile(id, file);
        return ResponseEntity.ok().build();
    }
}
