package com.example.demo.initializers;

import com.example.demo.models.Account;
import com.example.demo.security.userdetails.UserDetailsServiceImpl;
import com.example.demo.services.impl.AccountServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@RequiredArgsConstructor
public class AdminInitializer implements CommandLineRunner {

    private final AccountServiceImpl service;
    private final UserDetailsServiceImpl userDetailsService;

    @Value("${admin.username}")
    private String username;

    @Value("${admin.password}")
    private String password;

    @Override
    public void run(String... args) {
        try {
            UserDetails admin = userDetailsService.loadUserByUsername(username);
            log.info("Admin " + admin.getUsername() + " already created, skipping");
        } catch (UsernameNotFoundException e) {
            Account admin = new Account(
                    username,
                    password,
                    "ADMIN_ROLE"
            );

            service.register(admin);

            log.info("Admin created successfully");
        }
    }
}
