package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Account;
import com.example.demo.models.Storage;
import com.example.demo.repositories.AccountRepository;
import com.example.demo.security.userdetails.UserDetailsImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class AccountServiceImpl {
    private final AccountRepository repository;
    private final PasswordEncoder passwordEncoder;
    private final StorageServiceImpl storageService;

    @Transactional
    public void register(Account account) {
        if (repository.findByEmail(account.getEmail()).isPresent()) {
            throw new ApiException("Пользователь с таким email уже существует");
        }

        String encodedPassword = passwordEncoder.encode(account.getPassword());
        account.setPassword(encodedPassword);

        Storage storage = storageService.createDefault();
        storage.setOwner(account);
        account.setStorage(storage);

        repository.save(account);
    }

    @Transactional
    public void updatePassword(String password) {
        Account existingAccount = getAuthenticated();
        existingAccount.setPassword(passwordEncoder.encode(password));
        repository.save(existingAccount);
    }

    public Account getAuthenticated() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.isAuthenticated()) {
            throw new ApiException("Пользователь не аутентифицирован");
        }

        return ((UserDetailsImpl)authentication.getPrincipal()).getAccount();
    }
}