package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Account;
import com.example.demo.models.Storage;
import com.example.demo.repositories.AccountRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class AdminAccountServiceImpl {
    private final AccountRepository repository;
    private final PasswordEncoder passwordEncoder;
    private final StorageServiceImpl storageService;

    public Account getById(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new ApiException("Аккаунт не найден"));
    }

    @Transactional
    public void register(Account account, Long diskSize) {
        if (repository.findByEmail(account.getEmail()).isPresent()) {
            throw new ApiException("Пользователь с таким email уже существует");
        }

        String encodedPassword = passwordEncoder.encode(account.getPassword());
        account.setPassword(encodedPassword);

        Storage storage = storageService.create(diskSize);
        storage.setOwner(account);
        account.setStorage(storage);

        repository.save(account);
    }

    @Transactional
    public void edit(Long id, Account updatedAccount, Long diskSize) {
        Account account = getById(id);

        if (repository.findByEmail(updatedAccount.getEmail()).isPresent()) {
            throw new ApiException("Пользователь с таким email уже существует");
        }

        String encodedPassword = passwordEncoder.encode(updatedAccount.getPassword());
        account.setPassword(encodedPassword);

        account.getStorage().setFreeSpaceInMB(diskSize);
        account.setRole(updatedAccount.getRole());

        repository.save(account);
    }

    @Transactional
    public void delete(Long id) {
        if (repository.existsById(id)) {
            repository.deleteById(id);
        }
    }
}
