package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Directory;
import com.example.demo.repositories.DirectoryRepository;
import com.example.demo.repositories.FileRepository;
import lombok.RequiredArgsConstructor;
import org.hibernate.Hibernate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class AdminDirectoryServiceImpl {

    private final AdminAccountServiceImpl accountService;
    private final DirectoryRepository directoryRepository;
    private final FileRepository fileRepository;

    public Directory findRoot(Long userId) {
        return accountService.getById(userId).getStorage().getDirectories()
                .stream()
                .filter(d -> d.getParent() == null)
                .findAny()
                .orElseThrow(() -> new ApiException("Корневая папка не найдена"));
    }

    public Directory findDirectoryById(Long id) {
        Directory directory = directoryRepository.findById(id)
                .orElseThrow(()->new ApiException("Папка не найдена"));

        Hibernate.initialize(directory.getFiles());

        return directory;
    }

    public List<Directory> findChildren(Long dirId) {
        return findDirectoryById(dirId).getChildren();
    }

    @Transactional
    public void createDirectory(String name, Long parentId) {
        Directory parent = findDirectoryById(parentId);

        Directory directory = new Directory();
        directory.setStorage(parent.getStorage());
        directory.setName(name);
        directory.setParent(parent);

        directoryRepository.save(directory);
    }

    @Transactional
    public void updateDirectory(Long id, String name, Long parentId) {

        Directory directory = findDirectoryById(id);
        Directory parent = findDirectoryById(parentId);

        directory.setName(name);
        directory.setParent(parent);

        directoryRepository.save(directory);
    }

    @Transactional
    public void deleteDirectory(Long id) {
        Directory directory = findDirectoryById(id);
        deleteAllFilesRecursive(directory);
        deleteAllDirectoriesRecursive(directory);
    }

    @Transactional
    public void deleteAllFilesRecursive(Directory directory) {
        directory.getFiles().forEach(fileRepository::delete);
        if (!directory.getChildren().isEmpty()) {
            directory.getChildren().forEach(this::deleteAllFilesRecursive);
        }
    }

    @Transactional
    public void deleteAllDirectoriesRecursive(Directory directory) {
        if (!directory.getChildren().isEmpty()) {
            directory.getChildren().forEach(this::deleteAllDirectoriesRecursive);
        }

        directoryRepository.delete(directory);
    }
}
