package com.example.demo.services.impl;

import com.example.demo.models.Directory;
import com.example.demo.models.Storage;
import com.example.demo.repositories.StorageRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class StorageServiceImpl {

    private final StorageRepository storageRepository;
    private final DirectoryServiceImpl directoryService;

    @Value("${storage.default.freeSpace}")
    private Long defaultFreeSpace;

    @Transactional
    public Storage createDefault() {
        return create(defaultFreeSpace);
    }

    @Transactional
    public Storage create(Long diskSize) {
        Storage storage = new Storage();
        storage.setFreeSpaceInMB(diskSize);

        Directory root = directoryService.createRoot();
        root.setStorage(storage);

        List<Directory> directories = new ArrayList<>();
        directories.add(directoryService.createRoot());
        storage.setDirectories(directories);

        return storageRepository.save(storage);
    }
}
